//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.CustomGrouping
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			Worker worker1 = new Worker();
			worker1.Name = "Jim";
			Worker worker2 = new Worker();
			worker2.Name = "Paul";
			Worker worker3 = new Worker();
			worker3.Name = "Kim";
			Worker worker4 = new Worker();
			worker4.Name = "Kate";
			Worker worker5 = new Worker();
			worker5.Name = "Chris";
			Worker worker6 = new Worker();
			worker6.Name = "Oliver";

			Machine machine1 = new Machine();
			machine1.Name = "Machine #1";
			machine1.Workers.Add(worker1);
			machine1.Workers.Add(worker3);
			Machine machine2 = new Machine();
			machine2.Name = "Machine #2";
			machine2.Workers.Add(worker2);
			machine2.Workers.Add(worker5);
			machine2.Workers.Add(worker6);
			Machine machine3 = new Machine();
			machine3.Name = "Machine #3";
			machine3.Workers.Add(worker3);
			machine3.Workers.Add(worker4);
			machine3.Workers.Add(worker5);

			calendar.Resources.Add(worker1);
			calendar.Resources.Add(worker2);
			calendar.Resources.Add(worker3);
			calendar.Resources.Add(worker4);
			calendar.Resources.Add(worker5);
			calendar.Resources.Add(worker6);
			calendar.Resources.Add(machine1);
			calendar.Resources.Add(machine2);
			calendar.Resources.Add(machine3);

			calendar.CustomizeGrouping += OnCustomizeGrouping;

			calendar.GroupType = GroupType.GroupByResources;
			calendar.SecondaryGroupType = GroupType.GroupByResources;
			calendar.CustomGroupType = typeof(Machine);
			calendar.CustomSecondaryGroupType = typeof(Worker);

			calendar.CustomDraw = CustomDrawElements.ItemMoveBand;
			calendar.Draw += OnDraw;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void OnCustomizeGrouping(object sender, CustomizeGroupingEventArgs e)
		{
			Machine machine = e.Parents[0] as Machine;
			e.GroupByResources.Clear();
			foreach (Worker worker in machine.Workers)
				e.GroupByResources.Add(worker);
		}

		private void OnDraw(object sender, DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.ItemMoveBand)
			{
				// Find the worker associated with the item
				Worker worker = null;
				foreach (Resource resource in e.Item.Resources)
				{
					worker = resource as Worker;
					if (worker != null)
						break;
				}

				// Find all items associated with this worker
				var items = calendar.Schedule.GetAllItems(e.Item.StartTime, e.Item.EndTime, worker);
				if (items.Count > 1)
				{
					Rectangle bounds = e.Bounds;
					bounds.X += 1;
					bounds.Y += 1;
					bounds.Width -= 1;
					bounds.Height -= 1;
					e.Graphics.FillRectangle(Brushes.Red, bounds);
				}
			}
		}


		private string _appPath;
	}

	public class Worker : Resource
	{
		public Worker()
		{
		}

		public Image Image
		{
			get { return image; }
			set { image = value; }
		}


		private Image image;
	}

	public class Machine : Resource
	{
		public Machine()
		{
			workers = new List<Worker>();
		}

		public Image Image
		{
			get { return image; }
			set { image = value; }
		}

		public List<Worker> Workers
		{
			get { return workers; }
		}


		private Image image;
		private List<Worker> workers;
	}
}